@@ -150,7 +150,7 @@ foreach (qw(sites basedirs patterns)) {
 foreach (qw(parse_result search_result)) {
     has $_ => (is => 'rw', default => sub { {} });
 }
-foreach (qw(upstream_url newfile_base)) {
+foreach (qw(force_repack type upstream_url newfile_base)) {
     has $_ => (is => 'rw');
 }
 
@@ -200,6 +200,8 @@ has headers => (
     },
 );
 
+my $minversion = '';
+
 ###############
 # Main method #
 ###############
@@ -311,6 +313,22 @@ following keys:
 # greatest version number (as determined by the (...) group), using the
 # Debian version number comparison algorithm described below.
 
+sub BUILD {
+    my ($self, $args) = @_;
+    if ($self->watch_version > 3) {
+        my $line = $self->line;
+        if ($line =~ s/^opt(?:ion)?s\s*=\s*//) {
+            unless ($line =~ s/^".*?"(?:\s+|$)//) {
+                $line =~ s/^[^"\s]\S*(?:\s+|$)//;
+            }
+        }
+        my ($base, $filepattern, $lastversion, $action) = split /\s+/, $line,
+          4;
+        $self->type($lastversion);
+    }
+    return $self;
+}
+
 sub parse {
     my ($self) = @_;
     uscan_debug "parse line $self->{line}";
@@ -396,7 +414,8 @@ EOF
                 or $opts =~ /^\s*useragent\s*=\s*(.+?)\s*$/) {
                 my $user_agent_string = $1;
                 $user_agent_string = $self->config->user_agent
-                  if $self->config->user_agent;
+                  if $self->config->user_agent ne
+                  &Devscripts::Uscan::Config::default_user_agent;
                 $self->downloader->user_agent->agent($user_agent_string);
                 uscan_verbose "User-agent: $user_agent_string";
                 $opts = '';
@@ -405,11 +424,11 @@ EOF
             foreach my $opt (@opts) {
                 uscan_verbose "Parsing $opt";
                 if ($opt =~ /^\s*pasv\s*$/ or $opt =~ /^\s*passive\s*$/) {
-                    $self->downloader->passive(1);
+                    $self->downloader->pasv(1);
                 } elsif ($opt =~ /^\s*active\s*$/
                     or $opt =~ /^\s*nopasv\s*$/
                     or $opt =~ /^s*nopassive\s*$/) {
-                    $self->downloader->passive(0);
+                    $self->downloader->pasv(0);
                 }
 
                 # Line option "compression" is ignored if "--compression"
@@ -531,16 +550,11 @@ EOF
         }
 
         # compression is persistent
-        $self->compression((
-                     $self->mode eq 'http'
-                  or $self->mode eq 'ftp'
-            )
-            ? get_compression('gzip')    # keep backward compat
-            : get_compression('xz')) unless ($self->compression);
+        $self->compression('default') unless ($self->compression);
 
         # Set $lastversion to the numeric last version
         # Update $self->versionmode (its default "newer")
-        if (!length($lastversion) or $lastversion eq 'debian') {
+        if (!length($lastversion) or $lastversion =~ /^(group|debian)$/) {
             if (!defined $self->pkg_version) {
                 uscan_warn "Unable to determine the current version\n"
                   . "  in $watchfile, skipping:\n  $self->{line}";
@@ -549,10 +563,10 @@ EOF
             $lastversion = $self->pkg_version;
         } elsif ($lastversion eq 'ignore') {
             $self->versionmode('ignore');
-            $lastversion = $self->config->minversion;
+            $lastversion = $minversion;
         } elsif ($lastversion eq 'same') {
             $self->versionmode('same');
-            $lastversion = $self->config->minversion;
+            $lastversion = $minversion;
         } elsif ($lastversion =~ m/^prev/) {
             $self->versionmode('previous');
 
@@ -712,7 +726,12 @@ EOF
 "specified --download-debversion to set the last version: $lastversion";
     } elsif ($self->versionmode eq 'previous') {
         $lastversion = $self->shared->{previous_newversion};
-        uscan_verbose "Previous version downloaded: $lastversion";
+        # $lastversion is set only if something was downloaded before
+        if ($lastversion) {
+            uscan_verbose "Previous version downloaded: $lastversion";
+        } else {
+            uscan_verbose "Previous version not set, skipping";
+        }
     } else {
         uscan_verbose
 "Last orig.tar.* tarball version (from debian/changelog): $lastversion";
@@ -770,9 +789,14 @@ EOF
     } elsif ($self->versionmode eq 'previous') {
         unless ($self->pgpmode eq 'previous'
             and defined $self->shared->{previous_newversion}) {
-            uscan_warn
+            if ($self->shared->{download}) {
+                uscan_warn
 "Unable to set versionmode=prev for the line without opts=pgpmode=prev\n"
-              . "  in $watchfile, skipping:\n  $self->{line}";
+                  . "  in $watchfile, skipping:\n  $self->{line}";
+            } else {
+                uscan_warn "Nothing was downloaded before, skipping pgp check";
+                uscan_verbose "  line " . $self->line;
+            }
             return $self->status(1);
         }
         $self->shared->{download_version}
@@ -917,7 +941,7 @@ sub search {
 
         # Old-style heuristics
         if ($newversion =~ /^\D*(\d+\.(?:\d+\.)*\d+)\D*$/) {
-            $newversion = $1;
+            $self->search_result->{newversion} = $1;
         } else {
             uscan_warn <<"EOF";
 $progname warning: In $self->{watchfile}, couldn\'t determine a
@@ -1145,6 +1169,7 @@ sub download_file_and_sig {
                 $self,
                 $self->parse_result->{base},
                 $self->pkg_dir,
+                $self->pkg,
                 $self->mode
             );
             if ($download_available) {
@@ -1168,6 +1193,7 @@ sub download_file_and_sig {
                 $self,
                 $self->parse_result->{base},
                 $self->pkg_dir,
+                $self->pkg,
                 $self->mode,
             );
             if ($download_available) {
@@ -1251,8 +1277,7 @@ sub download_file_and_sig {
                     return $self->status(1);
                 }
             } else {
-                uscan_warn "Unknown type file to decompress: $sigfile_base";
-                exit 1;
+                uscan_die "Unknown type file to decompress: $sigfile_base";
             }
         }
     }
@@ -1317,7 +1342,8 @@ sub download_file_and_sig {
                 $self->downloader->download(
                     $pgpsig_url,    "$self->{config}->{destdir}/$sigfile",
                     $self,          $self->parse_result->{base},
-                    $self->pkg_dir, $self->mode
+                    $self->pkg_dir, $self->pkg,
+                    $self->mode
                 ));
         } else {    # -1, 0
             uscan_verbose "Not downloading OpenPGP signature from\n"
@@ -1335,7 +1361,8 @@ sub download_file_and_sig {
                 $self->downloader->download(
                     $pgpsig_url,    "$self->{config}->{destdir}/$sigfile",
                     $self,          $self->parse_result->{base},
-                    $self->pkg_dir, $self->mode
+                    $self->pkg_dir, $self->pkg,
+                    $self->mode
                 ));
         } else {    # -1, 0
             uscan_verbose "Not downloading OpenPGP signature from\n"
@@ -1414,7 +1441,7 @@ sub download_file_and_sig {
         } else {
             my ($gitrepo, $gitref) = split /[[:space:]]+/, $self->upstream_url;
             $self->keyring->verify_git($self->pkg . "-temporary.$$.git",
-                $gitref);
+                $gitref, $self->downloader->git_upstream);
         }
     } else {
         uscan_warn "strange ... unknown pgpmode = $self->{pgpmode}";
@@ -1484,44 +1511,42 @@ sub mkorigtargz {
     my $path   = "$self->{config}->{destdir}/$self->{newfile_base}";
     my $target = $self->newfile_base;
     unless ($self->symlink eq "no") {
-        my @cmd = ("mk-origtargz");
-        push @cmd, "--package", $self->pkg;
-        push @cmd, "--version", $self->shared->{common_mangled_newversion};
-        push @cmd, '--repack-suffix', $self->repacksuffix
+        require Devscripts::MkOrigtargz;
+        @ARGV = ();
+        push @ARGV, "--package", $self->pkg;
+        push @ARGV, "--version", $self->shared->{common_mangled_newversion};
+        push @ARGV, '--repack-suffix', $self->repacksuffix
           if $self->repacksuffix;
-        push @cmd, "--rename" if $self->symlink eq "rename";
-        push @cmd, "--copy"   if $self->symlink eq "copy";
-        push @cmd, "--signature", $self->signature_available
+        push @ARGV, "--rename" if $self->symlink eq "rename";
+        push @ARGV, "--copy"   if $self->symlink eq "copy";
+        push @ARGV, "--signature", $self->signature_available
           if ($self->signature_available != 0);
-        push @cmd, "--signature-file",
+        push @ARGV, "--signature-file",
           "$self->{config}->{destdir}/$self->{search_result}->{sigfile}"
           if ($self->signature_available != 0);
-        push @cmd, "--repack" if $self->repack;
-        push @cmd, "--component", $self->component
+        push @ARGV, "--repack"       if $self->repack;
+        push @ARGV, "--force-repack" if $self->force_repack;
+        push @ARGV, "--component", $self->component
           if $self->component;
-        push @cmd, "--compression",    $self->compression;
-        push @cmd, "--directory",      $self->config->destdir;
-        push @cmd, "--copyright-file", "debian/copyright"
+        push @ARGV, "--compression",    $self->compression;
+        push @ARGV, "--directory",      $self->config->destdir;
+        push @ARGV, "--copyright-file", "debian/copyright"
           if ($self->config->exclusion && -e "debian/copyright");
-        push @cmd, "--copyright-file", $self->config->copyright_file
+        push @ARGV, "--copyright-file", $self->config->copyright_file
           if ($self->config->exclusion && $self->config->copyright_file);
-        push @cmd, "--unzipopt", $self->unzipopt
+        push @ARGV, "--unzipopt", $self->unzipopt
           if $self->unzipopt;
-        push @cmd, $path;
+        push @ARGV, $path;
+        my $tmp = $Devscripts::Output::die_on_error;
 
-        my $actioncmd = join(" ", @cmd);
-        uscan_verbose "Executing internal command:\n   $actioncmd";
-        spawn(
-            exec       => \@cmd,
-            to_string  => \$mk_origtargz_out,
-            wait_child => 1
-        );
-        chomp($mk_origtargz_out);
-        $path = $1
-          if $mk_origtargz_out
-          =~ /Successfully .* (?:to|as) ([^,]+)(?:,.*)?\.$/;
-        $path = $1 if $mk_origtargz_out =~ /Leaving (.*) where it is/;
-        $target = basename($path);
+        uscan_verbose "Launch mk-origtargz with options:\n   "
+          . join(" ", @ARGV);
+        my $mk = Devscripts::MkOrigtargz->new;
+        $mk->do;
+        uscan_die "mk-origtargz failed" if ($mk->status);
+
+        $path                                      = $mk->destfile_nice;
+        $target                                    = basename($path);
         $self->shared->{common_mangled_newversion} = $1
           if $target =~ m/[^_]+_(.+)\.orig(?:-.+)?\.tar\.(?:gz|bz2|lzma|xz)$/;
         uscan_verbose "New orig.tar.* tarball version (after mk-origtargz): "
